# Repository path:    $HeadURL: https://90.145.156.222/DMS-Houtribdijk/code/nc-to-thredds/source/jspect.py $
# Last committed:     $Revision: 380303 $; $Author: vvuik $; $Date: 2019-10-21 11:46:46 +0200 (ma, 21 okt 2019) $
#
# -*- coding: utf-8 -*-
"""
Created on Wed May 29 09:24:48 2019

@author: vuik
"""

import numpy as np
from scipy import signal
# from windowFunctions import hann_local

def jspect(X,Y,N,DT=1,DW='hann',OVERLAP=0,DETREND=1):

    # function F,P=jspect(X,Y,N,DT,DW,OVERLAP,DETREND)
    #
    # JSPECT (Similiar to CROSGK), but uses scipy.signal.csd  
    # 
    # Usage: F,P=jspect(X,Y,N,DT,DW,OVERLAP,DETREND)
    #
    # Input:
    # X  contains the data of series 1
    # Y  contains the data of series 2
    # N  is the number of samples per data segment (power of 2)
    # DT is the time step (optional), default DT=1 s
    # DW is the data window type (optional): DW = 1 for hann window (default)
    #                                        DW = 2 for rectangular window
    #                                        DW = 3 for bartlett window
    #                                        DW = 4 for blackman window
    #                                        DW = 5 for hamming window
    # specification of scipy window types is recommended
    #
    # OVERLAP(optional) : 0=0% (default) 1=50% overlap, etc
    # DETREND(optional) : 1 = linear(default), 2 = quadratic;
    # Output:
    # P contains the (cross-)spectral estimates: list element 1 = Pxx, 2 = Pyy, 3 = Pxy
    # F contains the frequencies at which P is given
    #
    # JAMIE MACMAHAN 19/6/2002
    # Matlab to Python: Vincent Vuik, 29/5/2019
    
    # Window selection
    if DW=='hann' or DW==1:
        # from windowFunctions import hann_local
        # win = hann_local(N) # specify win vector by function hann_local
        DOF = np.floor(8/3*np.size(X)/(N/2))
        win = 'hann'
    elif DW=='rectwin' or DW==2:
        # from windowFunctions import rectwin
        # win = rectwin(N)
        DOF = np.floor(np.size(X)/(N/2))
        win = 'rectwin' # not an option in python! if needed: implement in windowFunctions.py
    elif DW=='bartlett' or DW==3:
        # from windowFunctions import bartlett
        # win = bartlett(N)
        DOF = np.floor(3*np.size(X)/(N/2))
        win = 'bartlett'
    elif DW=='blackman' or DW==4:
        # from windowFunctions import blackman
        # win = blackman(N)
        DOF = np.floor(np.size(X)/(N/2))
        win = 'blackman'
    elif DW=='hamming' or DW==5:
        # from windowFunctions import hamming
        # win = hamming(N)
        DOF = np.floor(2.5164*np.size(X)/(N/2))
        win = 'hamming'
    else:
        DOF = np.nan
        win = DW # choose a valid name of python window type
    
    
    # Original detrend code 19/06/2002
    if DETREND==1: # linear detrend
        X = signal.detrend(X)
        Y = signal.detrend(Y)
    elif DETREND==2: # quadratically detrend
        eps = 1e-9
        t  = np.arange(0,(DT*np.size(X))-DT+eps,DT)
        P1 = np.polyfit(t,X,2)
        P2 = np.polyfit(t,Y,2)
        X  = X-np.polyval(P1,t)
        Y  = Y-np.polyval(P2,t) # Matlab code: P1, check when DETREND=2 is to be used
    # Alternative: detrend per block (see Matlab code)
    
    # Estimate the cross power spectral density, Pxy, using Welch’s method
    Fs = 1/DT # sampling frequency
    fxx, Pxx = signal.csd(X,X,window=win,noverlap=OVERLAP,nperseg=N,fs=Fs)
    fyy, Pyy = signal.csd(Y,Y,window=win,noverlap=OVERLAP,nperseg=N,fs=Fs) 
    fxy, Pxy = signal.csd(X,Y,window=win,noverlap=OVERLAP,nperseg=N,fs=Fs)
    F   = fxx[0:-1]
    Pxx = Pxx[0:-1] # P(:,1)
    Pyy = Pyy[0:-1] # P(:,2)
    Pxy = Pxy[0:-1] # P(:,3)
            
    # Variance Perserving   
    fc1 = np.trapz(Pxx,F) # trapz y,x !
    fc2 = np.trapz(Pyy,F) # trapz y,x !
    fc3 = np.sqrt(fc1*fc2)
    Pxx = np.var(X)/fc1*Pxx   
    Pyy = np.var(Y)/fc2*Pyy
    Pxy = np.sqrt(np.var(Y)*np.var(X))/fc3*Pxy
    
    df = F[1]-F[0]
    
    # output spectrum characteristics
    print('number of samples used : ' + str(np.size(X)))
    print('degrees of freedom     : ' + str(DOF))
    print('resolution             : ' + str(df))
    
    P = [Pxx,Pyy,Pxy]
    
    return F,P
    
